import asyncio

import pytest_asyncio

import agp_bindings


@pytest_asyncio.fixture(scope="function")
async def server(request):
    # create new server
    global svc_server
    svc_server = await agp_bindings.create_pyservice("cisco", "default", "server")

    # configure it
    svc_server.configure(agp_bindings.GatewayConfig(endpoint=request.param, insecure=True))

    # init tracing
    agp_bindings.init_tracing(log_level="info")

    # run gateway server in background
    await agp_bindings.serve(svc_server)

    # wait for the server to start
    await asyncio.sleep(1)

    # return the server
    yield svc_server



# Copyright AGNTCY Contributors (https://github.com/agntcy)
# SPDX-License-Identifier: Apache-2.0

import argparse
import asyncio
import datetime
import os

from common import format_message, split_id

import agp_bindings
from agp_bindings import GatewayConfig


async def run_client(local_id, remote_id, address, producer, enable_opentelemetry: bool):
    # init tracing
    agp_bindings.init_tracing(log_level="info", enable_opentelemetry=enable_opentelemetry)

    # Split the IDs into their respective components
    local_organization, local_namespace, local_agent = split_id(local_id)

    # create new gateway object
    gateway = await agp_bindings.Gateway.new(local_organization, local_namespace, local_agent)

    # Configure gateway
    config = GatewayConfig(endpoint=address, insecure=True)
    gateway.configure(config)

    # Connect to the service and subscribe for the local name
    print(format_message("connecting to:", address))
    _ = await gateway.connect()

    # Split the IDs into their respective components
    remote_organization, remote_namespace, broadcast_topic = split_id(remote_id)

    # Get the local agent instance from env
    instance = os.getenv("AGP_INSTANCE_ID", local_agent)

    async with gateway:
        if producer:
            # Create a route to the remote ID
            await gateway.set_route(remote_organization, remote_namespace, broadcast_topic)

            # create streaming session with default config
            session_info = await gateway.create_streaming_session(
                agp_bindings.PyStreamingConfiguration(
                    agp_bindings.PySessionDirection.SENDER,
                    topic=None,
                    max_retries=5,
                    timeout=datetime.timedelta(seconds=5),
                )
            )

            # initialize count
            count = 0

            while True:
                try:
                    # Send a message
                    print(
                        format_message(
                            f"{instance} streaming message to {remote_organization}/{remote_namespace}/{broadcast_topic}: ",
                            f"{count}",
                        )
                    )

                    # Send the message and wait for a reply
                    await gateway.publish(
                        session_info,
                        f"{count}".encode(),
                        remote_organization,
                        remote_namespace,
                        broadcast_topic,
                    )

                    count += 1
                except Exception as e:
                    print("received error: ", e)
                finally:
                    await asyncio.sleep(1)
        else:
            # subscribe to streaming session
            await gateway.subscribe(remote_organization, remote_namespace, broadcast_topic)

            # Wait for messages and not reply
            while True:
                session_info, _ = await gateway.receive()
                print(
                    format_message(
                        f"{instance.capitalize()} received a new session:",
                        f"{session_info.id}",
                    )
                )

                async def background_task(session_info):
                    while True:
                        # Receive the message from the session
                        session, msg = await gateway.receive(session=session_info)
                        print(
                            format_message(
                                f"{local_agent.capitalize()} received from {remote_organization}/{remote_namespace}/{broadcast_topic}: ",
                                f"{msg.decode()}",
                            )
                        )

                asyncio.create_task(background_task(session_info.id))


async def main():
    parser = argparse.ArgumentParser(description="Command line client for message passing.")
    parser.add_argument(
        "-l",
        "--local",
        type=str,
        help="Local ID in the format organization/namespace/agent.",
    )
    parser.add_argument(
        "-r",
        "--remote",
        type=str,
        help="Remote ID in the format organization/namespace/agent.",
    )
    parser.add_argument(
        "-g",
        "--gateway",
        type=str,
        help="Gateway address.",
        default="http://127.0.0.1:46357",
    )
    parser.add_argument(
        "-t",
        "--enable-opentelemetry",
        action="store_true",
        default=False,
        help="Enable OpenTelemetry tracing.",
    )
    parser.add_argument(
        "-p",
        "--producer",
        action="store_true",
        default=False,
        help="Enable producer mode.",
    )

    args = parser.parse_args()

    # Run the client with the specified local ID, remote ID, and optional message
    await run_client(
        args.local,
        args.remote,
        args.gateway,
        args.producer,
        args.enable_opentelemetry,
    )


if __name__ == "__main__":
    try:
        asyncio.run(main())
    except KeyboardInterrupt:
        print("Program terminated by user.")



# This file is automatically generated by pyo3_stub_gen
# ruff: noqa: E501, F401

import builtins
import datetime
import typing
from enum import Enum, auto

class PyAgentType:
    r"""
    agent class
    """
    organization: builtins.str
    namespace: builtins.str
    agent_type: builtins.str
    def __new__(cls,agent_org:builtins.str, agent_ns:builtins.str, agent_class:builtins.str): ...

class PyFireAndForgetConfiguration:
    r"""
    fire and forget session config
    """
    def __new__(cls,): ...
    ...

class PyGatewayConfig:
    r"""
    gatewayconfig class
    """
    endpoint: builtins.str
    insecure: builtins.bool
    insecure_skip_verify: builtins.bool
    tls_ca_path: typing.Optional[builtins.str]
    tls_ca_pem: typing.Optional[builtins.str]
    tls_cert_path: typing.Optional[builtins.str]
    tls_key_path: typing.Optional[builtins.str]
    tls_cert_pem: typing.Optional[builtins.str]
    tls_key_pem: typing.Optional[builtins.str]
    basic_auth_username: typing.Optional[builtins.str]
    basic_auth_password: typing.Optional[builtins.str]
    def __new__(cls,endpoint:builtins.str, insecure:builtins.bool=False, insecure_skip_verify:builtins.bool=False, tls_ca_path:typing.Optional[builtins.str]=None, tls_ca_pem:typing.Optional[builtins.str]=None, tls_cert_path:typing.Optional[builtins.str]=None, tls_key_path:typing.Optional[builtins.str]=None, tls_cert_pem:typing.Optional[builtins.str]=None, tls_key_pem:typing.Optional[builtins.str]=None, basic_auth_username:typing.Optional[builtins.str]=None, basic_auth_password:typing.Optional[builtins.str]=None): ...

class PyRequestResponseConfiguration:
    r"""
    request response session config
    """
    max_retries: builtins.int
    timeout: builtins.int
    def __new__(cls,max_retries:builtins.int=0, timeout:builtins.int=1000): ...
    def set_max_retries(self, max_retries:builtins.int) -> None:
        ...

    def set_timeout(self, timeout:builtins.int) -> None:
        ...


class PyService:
    id: builtins.int
    def configure(self, config:PyGatewayConfig) -> None:
        ...


class PySessionInfo:
    id: builtins.int
    def __new__(cls,session_id:builtins.int): ...

class PyStreamingConfiguration:
    r"""
    streaming session config
    """
    direction: PySessionDirection
    topic: typing.Optional[PyAgentType]
    max_retries: builtins.int
    def __new__(cls,direction:PySessionDirection, topic:typing.Optional[PyAgentType], max_retries:typing.Optional[builtins.int]=None, timeout:typing.Optional[datetime.timedelta]=None): ...

class PySessionDirection(Enum):
    r"""
    session direction
    """
    SENDER = auto()
    RECEIVER = auto()
    BIDIRECTIONAL = auto()

def connect(svc:PyService) -> typing.Any:
    ...

def create_ff_session(svc:PyService, config:PyFireAndForgetConfiguration=...) -> typing.Any:
    ...

def create_rr_session(svc:PyService, config:PyRequestResponseConfiguration=...) -> typing.Any:
    ...

def create_streaming_session(svc:PyService, config:PyStreamingConfiguration) -> typing.Any:
    ...

def disconnect(svc:PyService, conn:builtins.int) -> typing.Any:
    ...

def publish(svc:PyService, session_info:PySessionInfo, fanout:builtins.int, blob:typing.Sequence[builtins.int], name:typing.Optional[PyAgentType]=None, id:typing.Optional[builtins.int]=None) -> typing.Any:
    ...

def receive(svc:PyService) -> typing.Any:
    ...

def remove_route(svc:PyService, conn:builtins.int, name:PyAgentType, id:typing.Optional[builtins.int]=None) -> typing.Any:
    ...

def serve(svc:PyService) -> typing.Any:
    ...

def set_route(svc:PyService, conn:builtins.int, name:PyAgentType, id:typing.Optional[builtins.int]=None) -> typing.Any:
    ...

def stop(svc:PyService) -> typing.Any:
    ...

def subscribe(svc:PyService, conn:builtins.int, name:PyAgentType, id:typing.Optional[builtins.int]=None) -> typing.Any:
    ...

def unsubscribe(svc:PyService, conn:builtins.int, name:PyAgentType, id:typing.Optional[builtins.int]=None) -> typing.Any:
    ...

